@@ -4,7 +4,6 @@ chain_name=aembit-redirect-chain
 aembit_group_id=$(id --group aembit_agent_proxy)
 agent_proxy_port=38080
 agent_dns_port=8053
-container_cidr=$AEMBIT_DOCKER_CONTAINER_CIDR
 
 update-alternatives --set iptables /usr/sbin/iptables-legacy
 update-alternatives --set ip6tables /usr/sbin/ip6tables-legacy
@@ -36,6 +35,19 @@ update-alternatives --set ip6tables /usr/sbin/ip6tables-legacy
 # for more information on conntrack.
 iptables -t raw -A OUTPUT -p udp --dport 53 -m owner --gid-owner "${aembit_group_id}" -j NOTRACK
 
+# The story for this rule is extremely similar to the above (only occurring in the VM environment).
+#
+# 1. CW used an ephemeral port to communicate with ResolveD (running on 127.0.0.53:53), and it's added to conntrack
+# 2. CW releases this port, and it becomes immediately available
+# 3. AP was accidentally given this port by the system when it tried to do DNS resolution as part of connecting to 
+# a server workload. 
+# 4. AP request to 127.0.0.53:53 is NOTRACK (per the rule above)
+# 5. ResolveD (127.0.0.53:53) response to AP hits a match in conntrack and NAT (for some reason, decides to modify 
+# resolved port), and this packet is unreachable.
+#
+# As a result, we are adding this rule to make sure that traffic from ResolveD is not NATed.
+iptables -t raw -A OUTPUT -p udp -s 127.0.0.53 --sport 53 -j NOTRACK
+
 iptables  -t nat -N "${chain_name}"
 
 # Ignore (return to the main chain) all packets from our agent proxy
@@ -44,8 +56,12 @@ iptables -t nat -A "${chain_name}" -m owner --gid-owner "${aembit_group_id}" -j
 # Ignore TCP loopback
 iptables -t nat -A "${chain_name}" -p tcp -o lo -j RETURN
 
-# Redirect everything else to agent proxy port port
-iptables -t nat -A "${chain_name}" -p  tcp -j REDIRECT --to-port "${agent_proxy_port}"
+# Redirect all new TCP connections to the Agent Proxy.
+# We use the "--syn" flag to forward new connections only, since,
+# prior to adding NAT rules, conntrack is not loaded. Once we add NAT rules,
+# conntrack is loaded, but mistakenly identifies existing connections as new
+# connections, redirecting them erroneously to the Agent Proxy mid-connection.
+iptables -t nat -A "${chain_name}" -p tcp --syn -j REDIRECT --to-port "${agent_proxy_port}"
 
 # Redirect DNS traffic to agent proxy DNS port, except for outbound traffic from systemd-resolved.
 resolve_d_user_id=$(id --user systemd-resolve);
@@ -56,8 +72,3 @@ iptables -t nat -A "${chain_name}" -p udp --dport 53 -j REDIRECT --to "${agent_d
 
 # Add out chain 
 iptables -t nat -A OUTPUT -j "${chain_name}"
-
-# If we know the containers' CIDR block, preroute container traffic through agent proxy
-if [ -n $container_cidr ]; then
-    iptables -t nat -A PREROUTING -p tcp -s $container_cidr -j REDIRECT --to-port $agent_proxy_port
-fi
